/*
 * Copyright (c) 2008 Motorola, Inc. or its subsidiaries. 
 * All rights reserved.
 *
 * DeviceStatusDashboard.js
 * This files contains:
 * - DeviceStatusDashboard class that represents the dashboard and its functionalities
 * - Device class that represents a device
 */



/**
 * Device Status Dashboard Class
 */
function DeviceStatusDashboard() {

	/*
	 * Constants
	 */
	
	var VALUE_UNKNOWN = -1;										// declaration to be accessible inside the object
	this.VALUE_UNKNOWN = -1;									// declaration to be accessible outside the object
	
	// Devices ids
	var DEVICE_BATTERY_ID = 'battery';
	var DEVICE_WIFI_ID = 'wifi';
	var DEVICE_SIGNAL_ID = 'signal';
	var DEVICE_DISK_ID = 'disk';
	var DEVICE_DATA_CONNECTION_ID = 'dataConnection';
	
	// Label of the volume that represents the phone memory
	var DISK_VOLUME_LABEL = '/dev/loop/0'; 
	
	// Key to set the preferences on widget object
	var ACTIVE_DEVICES_MAP_KEY = 'ACTIVE_DEVICES_MAP_KEY'; 		// declaration to be accessible inside the object
	this.ACTIVE_DEVICES_MAP_KEY = ACTIVE_DEVICES_MAP_KEY; 		// declaration to be accessible outside the object
	
	// Error messages
	var ERROR_BATTERY_INVALID = 'BATTERY<br>INVALID';
	var ERROR_BATTERY_NO = 'NO<br>BATTERY';
	var ERROR_BATTERY_STATUS_UNKNOWN = 'BATTERY<br>STATUS<br>UNKNOWN';
	var ERROR_WIFI_DISCONNECTED = 'WIFI<br>DISCONNECTED';
	var ERROR_WIFI_STATUS_UNKNOWN = 'WIFI<br>STATUS<br>UNKNOWN';
	var ERROR_SIGNAL_DISCONNECTED = 'CELLULAR SIGNAL<br>DISCONNECTED';
	var ERROR_SIGNAL_STATUS_UNKNOWN = 'SIGNAL<br>STATUS<br>UNKNOWN';
	var ERROR_DATA_DISCONNECTED = 'DATA LINK<br>DISCONNECTED';
	var ERROR_DATA_STATUS_UNKNOWN = 'DATA LINK<br>STATUS<br>UNKNOWN';	
	var ERROR_DISK_STATUS_UNKNOWN = 'DISK<br>STATUS<br>UNKNOWN';
	
	
	/*
	 * Properties
	 */
	 
	var devices = new Array(); // devices internal objects  
	var statusChangeListener = null; // status change listener. When any status changes, this hook will be invoked. 
	var activeDevicesMap; //object to store active devices


	
	/*
	 * 'Constructor'
	 */
	 
	activeDevicesMap = recoverActiveDevicesMap(); // recover the previous settings saved 
	activeDevicesMap.toString = function () { // overwrite the toString method to save the settings properly
		
		//The format is "<device id>=<true/false>" for each device, comma separated. 
		var value = '';
		
		value += DEVICE_BATTERY_ID + '=' + this[DEVICE_BATTERY_ID] + ',';
		value += DEVICE_WIFI_ID + '=' + this[DEVICE_WIFI_ID] + ',';
		value += DEVICE_SIGNAL_ID + '=' + this[DEVICE_SIGNAL_ID] + ',';
		value += DEVICE_DISK_ID + '=' + this[DEVICE_DISK_ID] + ',';
		value += DEVICE_DATA_CONNECTION_ID + '=' + this[DEVICE_DATA_CONNECTION_ID];
		
		return value;
	}
	initializeDashboard();
	
	
	
	/*
	 * Public Methods
	 */
	 
	/**
	 * Gets an array of Devices
	 * @return Array of Device objects
	 */
	this.getDevices = function() { 
		return devices;
	}
	
	/**
	 * Returns a Device object according to a device id
	 * @param id Device Id
	 * @return Device object
	 */
	this.getDeviceById = function(id) {
		return devices[id];
	}	
	
	/**
	 * Set the listener to be invoked when a status change
	 * @param fnct Function to be invoked
	 */
	this.setStatusChangeListener = function(fnct) {
		statusChangeListener = fnct;
	}
	
	/**
	 * Returns a map with active devices
	 * @return Map of active devices
	 */
	this.getActiveDevicesMap = function() {
		return activeDevicesMap;
	}
	
	
	/*
	 * Private Methods
	 */
	
	/**
	 *  Initialize Dashboard object with all devices information  
	 */
	function initializeDashboard() {
		
		var battery, wifi, signal, disk, dataConnection;
		var listener;
		
		// Battery
		battery = new Device();
		battery.name = 'Battery Power';
		battery.active = true;
		battery.liveconImageStatus = 'livecon_battery';
		battery.imageStatus = 'display_icon_battery';
		battery.imageStatusClass = 'batteryImage';
		battery.imageMenuSelected = 'resource/menu_icon_sel_battery.png';
		battery.imageMenuUnselected = 'resource/menu_icon_unsel_battery.png';
		battery.getStatus = getBatteryStatus;
		devices[DEVICE_BATTERY_ID]= battery;

		// Wifi network
		wifi = new Device();
		wifi.name = 'Wifi Signal';
		wifi.active = true;
		wifi.liveconImageStatus = 'livecon_wifi';
		wifi.imageStatus = 'display_icon_wifi';
		wifi.imageStatusClass = 'wifiImage';
		wifi.imageMenuSelected = 'resource/menu_icon_sel_wifi.png';
		wifi.imageMenuUnselected = 'resource/menu_icon_unsel_wifi.png';
		wifi.getStatus = getWifiStatus;
		devices[DEVICE_WIFI_ID]= wifi; 
		
		// Cellular signal		
		signal = new Device();
		signal.name = 'Cellular Signal';
		signal.active = true;
		signal.liveconImageStatus = 'livecon_signal';
		signal.imageStatus = 'display_icon_signal';
		signal.imageStatusClass = 'signalImage';
		signal.imageMenuSelected = 'resource/menu_icon_sel_signal.png';
		signal.imageMenuUnselected = 'resource/menu_icon_unsel_signal.png';
		signal.getStatus = getSignalStatus;
		devices[DEVICE_SIGNAL_ID]= signal;
		
		// Disk usage 		
		disk = new Device();
		disk.name = 'Disk Usage';
		disk.active = true;
		disk.liveconImageStatus = 'livecon_disk';
		disk.imageStatus = 'display_icon_disk';
		disk.imageStatusClass = 'diskImage';
		disk.imageMenuSelected = 'resource/menu_icon_sel_disk.png';
		disk.imageMenuUnselected = 'resource/menu_icon_unsel_disk.png';
		disk.getStatus = getFileSystemStatus;
		devices[DEVICE_DISK_ID]= disk;
				
		// Data connection
		dataConnection = new Device();
		dataConnection.name = 'Data Connection Status';
		dataConnection.active = true;
		dataConnection.liveconImageStatus = 'livecon_dataConnection';
		dataConnection.imageStatus = 'display_icon_dataConnection';
		dataConnection.imageStatusClass = 'dataConnectionImage';
		dataConnection.imageMenuSelected = 'resource/menu_icon_sel_dataConnection.png';
		dataConnection.imageMenuUnselected = 'resource/menu_icon_unsel_dataConnection.png';
		dataConnection.getStatus = getDataConnectionStatus;
		devices[DEVICE_DATA_CONNECTION_ID]= dataConnection;

		// Set the function that will handle changes in devices status
		listener = webui.status.createListener();
		listener.handle = handleStatusChange;
		webui.status.network.addListener(listener);
		webui.status.battery.addListener(listener);
		
		// Recover active devices previously saved
		if (widget.preferenceForKey(ACTIVE_DEVICES_MAP_KEY) != undefined) {
			 battery.active = activeDevicesMap[DEVICE_BATTERY_ID];
			 wifi.active = activeDevicesMap[DEVICE_WIFI_ID];
			 signal.active = activeDevicesMap[DEVICE_SIGNAL_ID];
			 disk.active = activeDevicesMap[DEVICE_DISK_ID];
			 dataConnection.active = activeDevicesMap[DEVICE_DATA_CONNECTION_ID];
		}

	}
	
	
	/**
	 *  Handles changes in devices status
	 *  @param event Event with the info of the device changed
	 */	
	function handleStatusChange(event) {
		
		// if statusChangeListener was defined (is not null)...
		if (statusChangeListener != null) { 
			
			// ... verify the status changed and invoke the listener
			switch (event.type) {
				case "status.battery.capacity" :
				case "status.battery.state" :
					statusChangeListener(DEVICE_BATTERY_ID);
					break;
				case "status.network.wifi.signalQuality" :
				case "status.network.wifi.state" :
					statusChangeListener(DEVICE_WIFI_ID);
					break;
				case "status.network.gsm.signalQuality" :
				case "status.network.gsm.state" :
					statusChangeListener(DEVICE_SIGNAL_ID);
					break;
				case "status.network.hsdpa.signalQuality" :
				case "status.network.hsdpa.state" :
					statusChangeListener(DEVICE_SIGNAL_ID);
					break;
				case "status.network.gprs.signalQuality" :
				case "status.network.gprs.state" :
					statusChangeListener(DEVICE_DATA_CONNECTION_ID);
					break;
				case "status.network.edge.signalQuality" :
				case "status.network.edge.state" :
					statusChangeListener(DEVICE_DATA_CONNECTION_ID);
					break;
			}
		}
	}
	

	/**
	 * Get Battery status
	 * @return Object with battery status information
	 */
	function getBatteryStatus() {
		var status = new Object();
		status.value = VALUE_UNKNOWN;
		status.label = '';
		status.error = '';
		
		switch (webui.status.battery.state) {
			// For NOT_CHARGEABLE, NORMAL and CHARGING, the capacity property is valid...
			case webui.status.defs.batteryState.NOT_CHARGEABLE:
			case webui.status.defs.batteryState.NORMAL:
			case webui.status.defs.batteryState.CHARGING:				
				status.value = webui.status.battery.capacity;
				if (status.value == VALUE_UNKNOWN){
					status.error = ERROR_BATTERY_STATUS_UNKNOWN;
				}
				break;
			// ... for the others, show an error message.
			case webui.status.defs.batteryState.NOT_VALID:
				status.error = ERROR_BATTERY_INVALID;
				break;
			case webui.status.defs.batteryState.NO_BATTERY:
				status.error = ERROR_BATTERY_NO;
				break;
			case webui.status.defs.batteryState.UNKNOWN:
			default:
				status.error = ERROR_BATTERY_STATUS_UNKNOWN;
				break;
		}

		return status;
	}
	
	
	/**
	 * Get Wifi Status
	 * @return Object with wifi status information
	 */
	function getWifiStatus() {
		var status = new Object();
		status.value = VALUE_UNKNOWN;
		status.label = '';
		status.error = '';
		
		switch (webui.status.network.wifi.state) {
			// For CONNECTED state, gets the signalQuality property...
			case webui.status.defs.networkState.CONNECTED:
				status.value = webui.status.network.wifi.signalQuality;
				if (status.value == VALUE_UNKNOWN){
					status.error = ERROR_WIFI_STATUS_UNKNOWN;
				}
				break;
			// ... for the others, show an error message.
			case webui.status.defs.networkState.DISCONNECTED:
				status.error = ERROR_WIFI_DISCONNECTED;
				break;
			case webui.status.defs.networkState.UNKNOWN:
			default: 
				status.error = ERROR_WIFI_STATUS_UNKNOWN;
				break;
		}
		
		return status;
	}
	
	
	/**
	 *  Get Signal Status
	 *	@return Object with cellular signal status information
	 */
	function getSignalStatus() {
		var status = new Object();
		status.value = VALUE_UNKNOWN;
		status.label = '';
		status.error = '';
		
		var gsm = webui.status.network.gsm.state;
		var hsdpa = webui.status.network.hsdpa.state;
		
		// If hsdpa is connected, gets its status...
		if ((hsdpa == webui.status.defs.networkState.CONNECTED)
			|| (hsdpa == webui.status.defs.networkState.TRANSMIT)) {
			status.value = webui.status.network.hsdpa.signalQuality;
			status.label = webui.status.network.hsdpa.name;
			if (status.value == VALUE_UNKNOWN){
				status.error = ERROR_SIGNAL_STATUS_UNKNOWN;
			}
		// ...else, if gsm is connected, gets its status...
		} else if (gsm == webui.status.defs.networkState.CONNECTED) {
			status.value = webui.status.network.gsm.signalQuality;
			status.label = webui.status.network.gsm.name;
			if (status.value == VALUE_UNKNOWN){
				status.error = ERROR_SIGNAL_STATUS_UNKNOWN;
			}
		// ... else, shows an error message.
		} else if (hsdpa == webui.status.defs.networkState.DISCONNECTED ||
					gsm == webui.status.defs.networkState.DISCONNECTED) {
			status.error = ERROR_SIGNAL_DISCONNECTED;
		} else {
			status.error = ERROR_SIGNAL_STATUS_UNKNOWN;
		}

		return status;
	}
	

	/**
	 * Get Data Connection Status
	 * @return Object with data connection status information
	 */
	function getDataConnectionStatus() {
		var status = new Object();
		status.value = VALUE_UNKNOWN;
		status.label = '';
		status.error = '';
		
		var gprs = webui.status.network.gprs.state;
		var edge = webui.status.network.edge.state;
		
		// If edge is connected, gets its status...
		if ((edge == webui.status.defs.dataNetworkState.ATTACHED)
			||(edge == webui.status.defs.dataNetworkState.CONNECT_SECURE)
			|| (edge == webui.status.defs.dataNetworkState.CONNECT_INSECURE)
			|| (edge == webui.status.defs.dataNetworkState.TRANSMIT)
			|| (edge == webui.status.defs.dataNetworkState.COVERAGE)) {
			status.value = webui.status.network.edge.signalQuality;
			status.label = webui.status.network.edge.name;
			if (status.value == VALUE_UNKNOWN){
				status.error = ERROR_DATA_STATUS_UNKNOWN;
			}
		// ... else, if gsm is connected, gets its status...
		} else if ((gprs == webui.status.defs.networkState.ATTACHED)
					|| (gprs == webui.status.defs.networkState.CONNECT_SECURE)
					|| (gprs == webui.status.defs.networkState.CONNECT_INSECURE)
					|| (gprs == webui.status.defs.networkState.TRANSMIT)){
			status.value = webui.status.network.gprs.signalQuality;
			status.label = webui.status.network.gprs.name;
			if (status.value == VALUE_UNKNOWN){
				status.error = ERROR_DATA_STATUS_UNKNOWN;
			}
		// ... else, shows an error message.
		} else if (edge == webui.status.defs.networkState.DISCONNECTED ||
					gprs == webui.status.defs.networkState.DISCONNECTED) {
			status.error = ERROR_DATA_DISCONNECTED;
		} else {
			status.error = ERROR_DATA_STATUS_UNKNOWN;
		}

		return status;
	}
	
	
	/**
	 * Get File System Status
	 * @return Object with file system status information
	 */
	function getFileSystemStatus() {
		
		var status = new Object();
		status.value = VALUE_UNKNOWN;
		status.label = '';
		status.error = '';
				
		var fileVolumeList = webui.filesystem.getVolumes(webui.filesystem.defs.volumeType.FS_ANY);
		var freeBytes, totalBytes;
		
		// Looks for the volume that represents the phone memory (DISK_VOLUME_LABEL)
		for (i = 0; i < fileVolumeList.length; i++) {
			if (fileVolumeList[i].label == DISK_VOLUME_LABEL) {
				freeBytes = fileVolumeList[i].freeBytes;
				totalBytes = fileVolumeList[i].totalBytes;
				break;
			}
		}
		
		// If the free and total Bytes are valid number, calculates the disk usage
		if (typeof freeBytes == 'number' && typeof totalBytes == 'number'
				&& totalBytes >= 0) {
			status.value = freeBytes/totalBytes;
			status.value = 1 - status.value;
			status.value = Math.round(status.value * 100);
		// Else, shows an error message
		} else {
			status.error = ERROR_DISK_STATUS_UNKNOWN;
		}

		return status;
	}	
	
	
	/**
	 * Recover active devices data from widget object
	 * @return Object to store active devices 
	 */
	function recoverActiveDevicesMap() {
		
		var pref = widget.preferenceForKey(ACTIVE_DEVICES_MAP_KEY);
		var obj = new Object();
		var devices = null;
		var key = null;
		var value = null;
		
		//If there is a key for active devices previously saved, parses its contents.
		//The format is "<device id>=<true/false>" for each device, comma separated. 
		if (pref != undefined) {
			devices = pref.split(',');
			
			for (i = 0; i < devices.length; i++) {
				key = devices[i].split('=')[0];
				value = devices[i].split('=')[1];
				
				obj[key] = (value == "true");
			}
		}
		
		return obj;
	}
}




/**
 * Device Class
 */
function Device() {

	this.name = null; // Device name
	this.active = null; // Active flage
	this.liveconImageStatus = null; // Image that represents the status on livecon mode
	this.imageStatus = null; // Image that represents the status on detailed mode
	this.imageStatusClass = null; // CSS class name for the imageStatus
	this.imageMenuSelected = null; // Image that represents the device selected on the menu
	this.imageMenuUnselected = null; // Image that represents the device unselected on the menu
	this.getStatus = null; // Function that returns the device status
}