/*
 * Copyright (c) 2008 Motorola, Inc. or its subsidiaries. 
 * All rights reserved.
 *
 * MotodevCarouselMenu.js
 * Component that implements a animated carousel menu for WebUI applications
 * This files contains:
 * - MotodevCarouselMenu class
 * - MotodevCarouselMenuItem class
 * - Functions to scroll the menu 
 */



/*
 * Exceptions
 */
var MOTODEV_MENU_NO_ITEMS_EXCEPTION = 'MOTODEV_MENU_NO_ITEMS_EXCEPTION';
var MOTODEV_MENU_INVALID_ITEM_EXCEPTION = 'MOTODEV_MENU_INVALID_ITEM_EXCEPTION';


 
/**
 * MotodevCarouselMenu Class
 */
function MotodevCarouselMenu() {

	/*
	 * Constants
	 */
	
	// Time to slide the menu (in miliseconds)
	var CAROUSEL_SLIDE_TIMEOUT = 30;

	// HTML Components
	var CAROUSEL_MENU_CENTER_ID = 'centerCarouselMenu';

	var CAROUSEL_MOBILE_DIV_ID = 'carouselMobileDiv';	
	
	// CSS Classes
	var CAROUSEL_MENU_CLASS = 'carouselMenu';
	var CAROUSEL_MENU_LEFT_SIDE_CLASS = 'leftCarouselMenu';
	var CAROUSEL_MENU_CENTER_CLASS = 'centerCarouselMenu';
	var CAROUSEL_MENU_RIGHT_SIDE_CLASS = 'rightCarouselMenu';

	var CAROUSEL_ITEM_SELECTED_CLASS = 'carouselItemSel';
	var CAROUSEL_IMG_ITEM_SELECTED_CLASS = 'carouselImgItemSel';

	var CAROUSEL_ITEM_UNSELECTED_CLASS = 'carouselItemUnsel';
	var CAROUSEL_IMG_ITEM_UNSELECTED_CLASS = 'carouselImgItemUnsel';
	
	var CAROUSEL_ITEM_PADDING = 3;
	


	/*
	 * Properties
	 */
	var previousImage; // image that represents the previous button
	var previousCallback; // callback to be executed on previous button click
	var nextImage; // image that represents the next button
	var nextCallback; // callback to be executed on next button click
	
	var menuItems = new Array(); // menu items internal objects
	var currentIndex; // current index selected on the menu
	var menuItemWidth, menuItemHeight; // width and height of each menu item
	var fakeDiv; // fake menu item
	
	var isMenuVisible = false;
	
	

	/*
	 * Public Methods
	 */
	 
	/**
	 * Returns the selected MotodevCarouselMenuItem object
	 * @return MotodevCarouselMenuItem Object
	 */	
	this.getCurrentItem = function () {

		// Verify if there is at least one item
		if (currentIndex == undefined) {
			throw MOTODEV_MENU_NO_ITEMS_EXCEPTION;
		}
		return menuItems[currentIndex];
	}


	/**
	 * Set carousel menu navigation images
	 * @param previous Left image.
	 * @param next Right image.
	 */	
	this.setNavigationImages = function(previous, next) {
		previousImage = previous;
		nextImage = next;
	}
	
	
	/**
	 * Set carousel menu navigation callbacks.
	 * On previous/next actions, the component will perform its
	 * actions, and execute a callback defined by component's user. 
	 * @param previous Callback to be executed on previous.
	 * @param next Callback to be executed on next.
	 */	
	this.setNavigationCallbacks = function(previous, next) {
		previousCallback = previous;
		nextCallback = next;
	}	
	

	/**
	 * Add item to the menu
	 * @param carouselMenuItem Object that represents a carousel menu item.
	 */	
	this.addItem = function(carouselMenuItem) {
		menuItems.push(carouselMenuItem);
		
		if (currentIndex == undefined) {
			currentIndex = 0;
		}
	}

	
	/**
	 * Set the menu visible or not
	 * @param visible Flag that indicates if menu is visible or not
	 */	
	this.setVisible = function (visible) {

		isMenuVisible = visible;
	}	


	/**
	 * Renders the menu inside an element
	 * @param elementId Id of the element that will contain the menu.
	 */	
	this.render = function(elementId) {
	
		var container;
		var table;
		var tr;
		var tdLeft, tdCenter, tdRight;
		var imgLeft, imgRight, imgDevice;
		var divImg;
		
		var divCenter, divFixed, divMobile; // divs to control menu scroll
		
		
		// If currentIndex is invalid, do nothing, else render menu
		if (currentIndex != undefined) {
		
			// Left arrow button		
			tdLeft = document.createElement('td'); 		
			tdLeft.className = CAROUSEL_MENU_LEFT_SIDE_CLASS;
			
			imgLeft = document.createElement('img');
			imgLeft.src = previousImage;
	
			tdLeft.appendChild(imgLeft);
			
			// Right arrow button
			tdRight = document.createElement('td'); 		
			tdRight.className = CAROUSEL_MENU_RIGHT_SIDE_CLASS;
	
			imgRight = document.createElement('img');
			imgRight.src = nextImage;
	
			tdRight.appendChild(imgRight);
			
			// Carousel menu items
			tdCenter = document.createElement('td');
			tdCenter.id = CAROUSEL_MENU_CENTER_ID;
			tdCenter.className = CAROUSEL_MENU_CENTER_CLASS;
			
			for (i in menuItems) { 
				// For each menu item, creates a div...
				divImg = document.createElement('div');
				
				// ... with an image for unselected item...
				imgDevice = document.createElement('img');
				imgDevice.className = CAROUSEL_IMG_ITEM_UNSELECTED_CLASS;
				imgDevice.src = menuItems[i].unselectedImage;
				divImg.appendChild(imgDevice);
				
				// ... and an image for selected item...
				imgDevice = document.createElement('img');
				imgDevice.className = CAROUSEL_IMG_ITEM_SELECTED_CLASS;
				imgDevice.src = menuItems[i].selectedImage;
				divImg.appendChild(imgDevice);
				
				// ... only one image will be shown, according to div's CSS class.
				divImg.className = CAROUSEL_ITEM_UNSELECTED_CLASS;
				divImg.width = menuItems[i].width + (CAROUSEL_ITEM_PADDING * 2);
				divImg.height = menuItems[i].height; 
				
				menuItems[i].htmlElement = divImg;
			}
	
			divMobile = document.createElement('div');	
			divMobile.id = CAROUSEL_MOBILE_DIV_ID;
			
			divFixed = document.createElement('div');
			divFixed.appendChild(divMobile);
			
			divCenter = document.createElement('div');
			divCenter.align = 'center';
			divCenter.appendChild(divFixed);
			
			tdCenter.appendChild(divCenter);
			
			// Render
			tr = document.createElement('tr');
			tr.appendChild(tdLeft);
			tr.appendChild(tdCenter);
			tr.appendChild(tdRight);
					
			table = document.createElement('table');
			table.className = CAROUSEL_MENU_CLASS;
			table.appendChild(tr);
			
			container = document.getElementById(elementId);
			container.innerHTML = '';
			container.appendChild(table);
			
			/*
			 * After renderization, correct properties for the divs that will be responsible for scrolling.
			 * The mobile div always need to have 5 items, and the width of these 5.
			 * The fixed div need to have the width of 3 item, and the overflow hidden.
			 * When scrolling, the mobile div is deslocated inside the fixed div.
			 */ 
			divMobile.style.position = 'absolute';

			// insert first element (current selected item)
			var currItem = getItem(currentIndex);
			divMobile.appendChild(currItem);
			
			// calculate the item width and height
			menuItemWidth = currItem.width;
	 		menuItemHeight = currItem.height;
	 		
	 		// create a fake blank div according to item width and height
	 		// this fake div will fill the menu positions was not supposed to show a menu item (the very
	 		// first and the very last ones), because the mobile div always need to have 5 items  
	 		fakeDiv = document.createElement('div');
	 		fakeDiv.style.width = (menuItemWidth - CAROUSEL_ITEM_PADDING * 2) + 'px';
	 		fakeDiv.style.height = (menuItemHeight - CAROUSEL_ITEM_PADDING * 2) + 'px';
	 		fakeDiv.style.padding = '0px ' + CAROUSEL_ITEM_PADDING + 'px';
	 		fakeDiv.style.display = 'inline';
	 		fakeDiv.style.position = 'relative';
	 		
	 		// insert the menu items 
	 		divMobile.insertBefore(getItem(currentIndex-1,'left'), divMobile.childNodes[0]);
	 		divMobile.insertBefore(getItem(currentIndex-2,'left'), divMobile.childNodes[0]);
	 		divMobile.appendChild(getItem(currentIndex+1,'right'));
			divMobile.appendChild(getItem(currentIndex+2,'right'));
			
			// set the selected item
			divMobile.childNodes[2].className = CAROUSEL_ITEM_SELECTED_CLASS;		
			
			// finish correcting other element's properties
	 		divMobile.style.width = (menuItemWidth * 5) + 'px';
			divMobile.style.height = menuItemHeight + 'px';
			divMobile.style.left = '-' + menuItemWidth + 'px';
			
			divFixed.style.width = (menuItemWidth * 3) + 'px' ;
			divFixed.style.height = menuItemHeight + 'px';
			divFixed.style.overflow = 'hidden'; 		
	 		
	 		divMobile.style.position = 'relative';
			divFixed.style.position = 'relative';
			
			isMenuVisible = true;
		
		} else {
			document.getElementById(elementId).innerHTML = '';
		}
	}
	
	
	/**
	 * Go to previous item. 
	 * Action to be executed when user select previous item. 
	 */	
	this.previous = function ()  {
		
		if (isMenuVisible) { // if menu is not visible, does not scroll items
		
			if (currentIndex != getPreviousIndex(currentIndex, false)) { // if there is a previous item
			
					if (!motodevCarouselMenuLOCK) { // check if scroll is lock (if there is an item scrolling already)
						motodevCarouselMenuLOCK = true;
						
						var mobileDiv = document.getElementById(CAROUSEL_MOBILE_DIV_ID);
						mobileDiv.childNodes[2].className = CAROUSEL_ITEM_UNSELECTED_CLASS;
						
						currentIndex = getPreviousIndex(currentIndex);
						
						// creates an object with all data necessary for scrolling
						var scrollData = new Object();
						scrollData.direction = 'right';
						scrollData.mobileDiv = mobileDiv;
						scrollData.itemSelectedClass = CAROUSEL_ITEM_SELECTED_CLASS;
						scrollData.slideTimeout = CAROUSEL_SLIDE_TIMEOUT;
						scrollData.menuItemWidth = menuItemWidth;
						scrollData.menuItemToInclude = getItem(currentIndex-2,'left');
						scrollData.callback = previousCallback;				
		
						// start scrolling thread
						MotodevCarouselScrollMenuStart(scrollData); 
					}
			}
			
			
		} else {
			currentIndex = getPreviousIndex(currentIndex, true);
		}
	}
	
	
	/**
	 * Go to next item. 
	 * Action to be executed when user select next item. 
	 */	
	this.next = function ()  {
		
		if (isMenuVisible) { // if menu is not visible, does not scroll items
		
				if (currentIndex != getNextIndex(currentIndex, false)) { // if there is a next item
		
							if (!motodevCarouselMenuLOCK) { // check if scroll is lock (if there is an item scrolling already)
								motodevCarouselMenuLOCK = true;
								
								var mobileDiv = document.getElementById(CAROUSEL_MOBILE_DIV_ID);
								mobileDiv.childNodes[2].className = CAROUSEL_ITEM_UNSELECTED_CLASS;
								
								currentIndex = getNextIndex(currentIndex);
								
								// creates an object with all data necessary for scrolling
								var scrollData = new Object();
								scrollData.direction = 'left';
								scrollData.mobileDiv = mobileDiv;
								scrollData.itemSelectedClass = CAROUSEL_ITEM_SELECTED_CLASS;
								scrollData.menuItemWidth = menuItemWidth;
								scrollData.slideTimeout = CAROUSEL_SLIDE_TIMEOUT;
								scrollData.menuItemToInclude = getItem(currentIndex+2,'right');
								scrollData.callback = nextCallback;
				
								// start scrolling thread
								MotodevCarouselScrollMenuStart(scrollData);			
							}
				}
		} else {
			currentIndex = getNextIndex(currentIndex, true);
		}
	}	
	
	
	
	/*
	 * Private Methods
	 */	
	
	/**
	 * Get a menu Item, according to the menu index.
	 * If an invalid index is passed, returns a fake menu item, according to position parameter
	 * @param index Index of the item
	 * @param position Position of the fake menu item (left/right)
	 * @returns Menu item.
	 */	
	function getItem(index,position)  {
		var item;
		
		if (menuItems[index] != undefined) {
			item = menuItems[index].htmlElement.cloneNode(true);
			item.width = menuItems[index].htmlElement.width;
			item.height = menuItems[index].htmlElement.height;
		} else {
			item = fakeDiv.cloneNode(true);
			
			if (position != undefined) item.style.cssFloat = position;
		}

		return item;
	}
			
	
	/**
	 * Get the previous index
	 * @param index Current Index
	 * @param isCircular Flag that indicates if the menu is circular or not
	 * @returns Previous index.
	 */	
	function getPreviousIndex(index, isCircular)  {
		var previous;
		
		if (index != undefined) {
			if (index == 0) {
				
				if (isCircular) previous = menuItems.length - 1;
				else previous = index;
				
			} else {
				previous = index - 1;
			}
		}

		return previous;
	}

	
	/**
	 * Get the next index
	 * @param index Current Index
	 * @param isCircular Flag that indicates if the menu is circular or not
	 * @returns Next index.
	 */	
	function getNextIndex(index, isCircular)  {
		var next;

		if (index != undefined) {
			if (index == menuItems.length - 1) {
				
				if (isCircular) next = 0;
				else next = index;				
								
			} else {
				next = index + 1;
			}
		}

		return next;
	}
}



/**
 * MotodevCarouselMenuItem Class
 *
 * @param pId Id of the menu item
 * @param pSelectedImage Image of the menu item to be shown when selected
 * @param pUnselectedImage Image of the menu item to be shown when unselected
 * @param pHeight Height of the menu item 
 * @param pWidth Width of the menu item
 */
function MotodevCarouselMenuItem( pId, pSelectedImage, pUnselectedImage, pHeight, pWidth) {

	// verifies if all properties were passed
	if (pId == null || pId == undefined ||
		 pSelectedImage == null || pSelectedImage == undefined ||
		 pUnselectedImage == null || pUnselectedImage == undefined ||
		 pHeight == null || pHeight == undefined ||
		 pWidth == null || pWidth == undefined) {
		 
		throw MOTODEV_MENU_INVALID_ITEM_EXCEPTION;
	}
	
	//attributes
	this.id = pId;
	this.selectedImage = pSelectedImage;
	this.unselectedImage = pUnselectedImage;
	this.height = pHeight;
	this.width = pWidth;
	
	this.htmlElement = null;
}



/*
 * These functions need to have global scope because they are used by interval threads  
 */

var motodevCarouselMenuData; // data necessary to do the scroll
var motodevCarouselMenuLOCK = false; // flag to lock any menu scrolling (only one item can scroll at time)

/**
 * Setup the initial data to scroll the menu
 * @parameter scrollData
 */
function MotodevCarouselScrollMenuStart(scrollData) {

	motodevCarouselMenuScrollData = scrollData;
	
	// set the initial scroll value variable
	motodevCarouselMenuScrollData.scrollPixels = motodevCarouselMenuScrollData.menuItemWidth * -1;

	// set the final scroll value variable 
	if (motodevCarouselMenuScrollData.direction == 'right')
		motodevCarouselMenuScrollData.scrollTotal = 0;	
	if (motodevCarouselMenuScrollData.direction == 'left')
		motodevCarouselMenuScrollData.scrollTotal = motodevCarouselMenuScrollData.menuItemWidth * -2;
	
	// start the scrolling thread
	motodevCarouselMenuScrollData.scrollInterval = setInterval("MotodevCarouselScrollMenu()", motodevCarouselMenuScrollData.slideTimeout);
}
	
/**
 * Scroll the menu 4 pixels. It is executed every CAROUSEL_SLIDE_TIMEOUT miliseconds until the menu
 * item is fully changed. 
 */
function MotodevCarouselScrollMenu(){
	
	// update the scroll variable
	if (motodevCarouselMenuScrollData.direction == 'right')
		motodevCarouselMenuScrollData.scrollPixels = motodevCarouselMenuScrollData.scrollPixels + 4;
	if (motodevCarouselMenuScrollData.direction == 'left')
		motodevCarouselMenuScrollData.scrollPixels = motodevCarouselMenuScrollData.scrollPixels - 4;
	
	// scroll mobile div
	motodevCarouselMenuScrollData.mobileDiv.style.left = motodevCarouselMenuScrollData.scrollPixels + "px";	

	// verify if scroll variable has reached its final value...
	if (motodevCarouselMenuScrollData.scrollPixels == motodevCarouselMenuScrollData.scrollTotal) {
		// ... if so, stop the scrolling thread and execute the scroll end
		clearInterval(motodevCarouselMenuScrollData.scrollInterval);
		MotodevCarouselScrollMenuEnd();
	}
}

/**
 * Finalize the menu scrolling
 */
function MotodevCarouselScrollMenuEnd() {
	
	// adjust the mobile div, removing the item that scrolled out of the bounds of the 5 items, and inserting
	// a new one...
	if (motodevCarouselMenuScrollData.direction == 'right') {
		// ...if scrolling to the right, remove the last item, and insert a new one in the first position 
		lastItemIndex = motodevCarouselMenuScrollData.mobileDiv.childNodes.length - 1;
		
		motodevCarouselMenuScrollData.mobileDiv.removeChild(
				motodevCarouselMenuScrollData.mobileDiv.childNodes[lastItemIndex]);
		
		motodevCarouselMenuScrollData.mobileDiv.insertBefore(motodevCarouselMenuScrollData.menuItemToInclude,
				motodevCarouselMenuScrollData.mobileDiv.childNodes[0]);
		
	} if (motodevCarouselMenuScrollData.direction == 'left') {
		// ...if scrolling to the left, remove the first item, and insert a new one in the last position
		motodevCarouselMenuScrollData.mobileDiv.removeChild(
				motodevCarouselMenuScrollData.mobileDiv.childNodes[0]);
		
		motodevCarouselMenuScrollData.mobileDiv.appendChild(motodevCarouselMenuScrollData.menuItemToInclude);			
	}
	
	// adjust the mobile div position and the selected item 
	motodevCarouselMenuScrollData.mobileDiv.style.left = '-' + motodevCarouselMenuScrollData.menuItemWidth + 'px';
	motodevCarouselMenuScrollData.mobileDiv.childNodes[2].className = motodevCarouselMenuScrollData.itemSelectedClass;
	
	
	// execute callback defined by component user
	if (motodevCarouselMenuScrollData.callback != undefined) {
		motodevCarouselMenuScrollData.callback();
	}
	
	motodevCarouselMenuScrollData = null;
	motodevCarouselMenuLOCK = false;
}