/*
 * Copyright (c) 2008 Motorola, Inc. or its subsidiaries. 
 * All rights reserved.
 *
 * main.js
 * This files contains the main functions, responsibles for navigation and events handling
 */

 

/*
 * Constants 
 */

var KEY_CODE_PREVIOUS = 37; // Key code for left soft key
var KEY_CODE_NEXT = 39; // Key code for right soft key
var KEY_CODE_UP = 38; // Key code for up soft key
var KEY_CODE_DOWN = 40; // Key code for down soft key
var KEY_CODE_CENTER = 13; // Key code for center soft key

var LIVECON_INTERVAL = 30000; // livecon interval in milliseconds

//HTML Components
var LIVECON_DIV = 'liveconView';
var DETAILED_DIV = 'detailedView';
var SETTINGS_DIV = 'settingsView';

var MENU_DIV = 'carouselMenuContainer';

var PAGE_TITLE = 'title';
var DEVICE_IMAGE = 'deviceImage';
var DEVICE_LABEL = 'deviceLabel';
var DEVICE_ERROR_LABEL = 'deviceErrorLabel';
var DEVICE_STATUS = 'deviceStatus';

var LIVECON_DEVICE_IMAGE = 'liveconDeviceImage';
var LIVECON_ERROR_LABEL = 'liveconDeviceErrorLabel';

var DEVICE_SETTINGS_DIV = 'deviceSettings';


/*
 * Global Variables
 */
var menu; // Object that implements the Motodev Carousel Menu
var deviceStatusDashboard; // Object that implements the Device Status Dashboard related functions

var liveconDiv; // Div that contains the Livecon View
var detailedDiv; // Div that contains the Detailed View
var settingsDiv; // Div that contains the Settings View

var liveconIntervalId = null; // interval id that controls livecon device flip;




/**
 * Function executed on page load. Does the initial setup. 
 */
function onLoadAction() {
	// Divs that represent each view/screen
	liveconDiv = document.getElementById(LIVECON_DIV);
	detailedDiv = document.getElementById(DETAILED_DIV);
	settingsDiv = document.getElementById(SETTINGS_DIV);
	
	// Set the actions on launch and on view switch
	webui.onLaunch = onLaunchAction;
	webui.onViewSwitch = onViewSwitchAction;
}


/**
 * Executed on launch. 
 */
function onLaunchAction() {

	// Initialize device dashboard object
	deviceStatusDashboard = new DeviceStatusDashboard();

	// Set the function that will handle changes in devices status
	deviceStatusDashboard.setStatusChangeListener(handleStatusChange);
	
	// Creates the menu object, using the devices in dashboard object
	createMenu();

	// Load widget according to current view (detailed or livecon)
	onViewSwitchAction();

}


/**
 * Executed on each view switch (between livecon and detailed views). 
 */
function onViewSwitchAction() {

	// If livecon thread (that flips the device on livecon view) is running, 
	// abort its execution, because it's switching from livecon to detailed view
	if (liveconIntervalId != null) {
		clearInterval(liveconIntervalId);
		liveconIntervalId = null;
	}
	
	switch (webui.viewType) {
	
		case webui.defs.viewType.LIVECON_VIEW:
			showLiveconView();
			break;
			
		case webui.defs.viewType.DETAILED_VIEW:			
			showDetailedView();
			break;
	}
}


/**
 * Show Livecon view
 */
function showLiveconView() {
	
	liveconDiv.style.display = 'block';
	detailedDiv.style.display = 'none';
	settingsDiv.style.display = 'none';
	
	// Resize livecon view div according to window size
	liveconDiv.style.height = window.innerHeight + 'px';
	liveconDiv.style.width = window.innerWidth + 'px';	
	
	// Update the displayed device status
	menu.setVisible(false);
	updateLiveconDeviceStatus();
	
	// Start thread that will flip the devices acording to the time defined in LIVECON_INTERVAL
	liveconIntervalId = setInterval("handleLiveconInterval()",LIVECON_INTERVAL);
}


/**
 * Show Detailed view
 */
function showDetailedView() {
	
	liveconDiv.style.display = 'none';
	detailedDiv.style.display = 'block';
	settingsDiv.style.display = 'none';
	
	// Resize detailed view div according to window size
	detailedDiv.style.height = window.innerHeight + 'px';
	detailedDiv.style.width = window.innerWidth + 'px';		

	// Update the displayed device status and render the menu object
	menu.render(MENU_DIV);
	updateDetailedDeviceStatus();
	
	// Set the key down event handles to implement the navigation on detailed view
	document.getElementById('body').setAttribute('onKeyDown','javascript:handleKeyDownMain(event);');
}


/**
 * Show Settings screen
 */
function showSettings() {
		
	var devices = deviceStatusDashboard.getDevices();
	var deviceSettings = document.getElementById(DEVICE_SETTINGS_DIV);
	var checkbox, label, div;
	
	liveconDiv.style.display = 'none';
	detailedDiv.style.display = 'none';
	settingsDiv.style.display = 'block';
	
	// Resize detailed view div according to window size
	settingsDiv.style.height = window.innerHeight + 'px';
	settingsDiv.style.width = window.innerWidth + 'px';
	
	deviceSettings.innerHTML = '';
	
	// Show a checkbox and a label for each device on dashboard object
	for (i in devices) {

		checkbox = document.createElement('img');
		checkbox.value = i;
		checkbox.checked = devices[i].active; 
		if (checkbox.checked) {
			checkbox.src = 'resource/checkbox_checked.png';
		} else {
			checkbox.src = 'resource/checkbox_unchecked.png';
		}
		checkbox.className = 'settingsCheckbox';

		label = document.createElement('label');
		label.innerHTML = '&nbsp;' + devices[i].name;
		label.className = 'settingsLabel';

		div = document.createElement('div');
		div.className = 'settingsItemDivUnselected';
		div.appendChild(checkbox);
		div.appendChild(label);
		
		deviceSettings.appendChild(div);
	}
	
	// Set the selected item
	deviceSettings.getElementsByTagName('div')[0].className = 'settingsItemDivSelected';
	
	// Set the key down event handles to implement the navigation on settings screen
	document.getElementById('body').setAttribute('onKeyDown','javascript:handleKeyDownSettings(event);');
}

/**
 * Save settings
 */
function saveSettings() {
	
	var devicesCheckBoxes = settingsDiv.getElementsByTagName('img');
	var devices = deviceStatusDashboard.getDevices();
	
	var deviceId;
	var activeDevicesMap = deviceStatusDashboard.getActiveDevicesMap();
	
	// Save the settings for each device on a object
	for (i = 0; i < devicesCheckBoxes.length; i++) {
		
		deviceId = devicesCheckBoxes[i].value;
		devices[deviceId].active = devicesCheckBoxes[i].checked;
		
		activeDevicesMap[deviceId] = devicesCheckBoxes[i].checked;
	}
	
	// Persist object with the data on widget object
	widget.setPreferenceForKey(activeDevicesMap, deviceStatusDashboard.ACTIVE_DEVICES_MAP_KEY);
	
	// Create the menu again, with the modified settings
	createMenu();
}


/**
 * Update device status on livecon view
 */
function updateLiveconDeviceStatus() {
	
	// HTML components variables
	var deviceImage = document.getElementById(LIVECON_DEVICE_IMAGE);
	var deviceErrorLabel = document.getElementById(LIVECON_ERROR_LABEL);
	
	// Gets the device status through selected menu item (invisible in livecon mode);
	var currentItem;
	var device;
	var status;
	
	try {
		currentItem = menu.getCurrentItem();
		device = deviceStatusDashboard.getDeviceById(currentItem.id);
		status = device.getStatus();
	
		if (status.value != deviceStatusDashboard.VALUE_UNKNOWN && status.value != undefined) { // if status is valid
			
			deviceErrorLabel.innerHTML = ''; // hide error message
			deviceImage.src = 'resource/' + device.liveconImageStatus + 
				getImageIndexByStatus(status.value) + '.png'; // show status image		
		
		} else { // if status is invalid
			
			// hide everything and show only the error message
			deviceImage.src = '';
			deviceImage.style.display = "none";
			deviceImage.style.display = "block";
			deviceErrorLabel.innerHTML = status.error;
		}	
		
	} catch (e) { // if there is no device available on menu, an exception is thrown
		if (e == MOTODEV_MENU_NO_ITEMS_EXCEPTION) {
			
			// hide everything and show only the error message
			deviceImage.src = '';
			deviceImage.style.display = "none";
			deviceImage.style.display = "block";
			deviceErrorLabel.innerHTML = 'No device selected';
		}
	}
}


/**
 * Update device status on detailed view
 */
function updateDetailedDeviceStatus() {
	
	// HTML components variables
	var titleElement = document.getElementById(PAGE_TITLE);
	var deviceImage = document.getElementById(DEVICE_IMAGE);
	var deviceLabel = document.getElementById(DEVICE_LABEL);
	var deviceErrorLabel = document.getElementById(DEVICE_ERROR_LABEL);
	var deviceStatus = document.getElementById(DEVICE_STATUS);
	
	// Gets the device status through selected menu item
	var currentItem;
	var device;
	var status;
	
	try {
		currentItem = menu.getCurrentItem();
		device = deviceStatusDashboard.getDeviceById(currentItem.id);
		status = device.getStatus();
		
		if (status.value != deviceStatusDashboard.VALUE_UNKNOWN && status.value != undefined) { // if status is valid
			
			deviceErrorLabel.innerHTML = ''; // hide error message
			deviceStatus.innerHTML = status.value + '%'; // show status value (in percentage)
			deviceLabel.innerHTML = status.label; // show status label
			deviceImage.className = device.imageStatusClass; // set proper class for status image
			deviceImage.src = 'resource/' + device.imageStatus + 
				getImageIndexByStatus(status.value) + '.png'; // show status image
		
		} else { // if status is invalid
			
			// hide everything and show only the error message
			deviceStatus.innerHTML = '';
			deviceLabel.innerHTML = '';
			deviceImage.src = '';
			deviceImage.className = '';
			deviceErrorLabel.innerHTML = status.error;
		}

		// Update page title
		titleElement.innerHTML = device.name;
		
	} catch (e) {  // if there is no device available on menu, an exception is thrown
		if (e == MOTODEV_MENU_NO_ITEMS_EXCEPTION) {
			
			// hide everything and show only the error message
			titleElement.innerHTML = '&nbsp;';
			deviceStatus.innerHTML = '';
			deviceLabel.innerHTML = '';
			deviceImage.src = '';
			deviceImage.className = '';
			deviceErrorLabel.innerHTML = 'Select a device<br>at settings screen.';
		}
	}
}


/**
 * Creates the devices menu, using Motodev Carousel Menu. 
 */
function createMenu() {
	
	var menuItem;
	var devices = deviceStatusDashboard.getDevices();
	
	menu = new MotodevCarouselMenu();
	
	// Iterate devices and create a menu item for each one of them
	for (i in devices) {
		if (devices[i].active) {
		
			menuItem = new MotodevCarouselMenuItem(i,devices[i].imageMenuSelected,
				devices[i].imageMenuUnselected,46,46);
			menu.addItem(menuItem);
		}
	}
	
	// Set the navigations images (left and right arrows)
	menu.setNavigationImages('resource/arrow_left.png','resource/arrow_right.png');

	// Set the callbacks to be executed on menu left and right actions
	menu.setNavigationCallbacks(updateDetailedDeviceStatus,updateDetailedDeviceStatus);
}


/**
 * Handles any key down on main screen. 
 * Implements the widget navigation through devices.
 *
 * @param event Object with the event key press
 */
function handleKeyDownMain(event) {

	switch(event.keyCode) {
	
		case KEY_CODE_PREVIOUS:
			menu.previous();
			break;
		case KEY_CODE_NEXT:
			menu.next();
			break;
	}
}

/**
 * Handles any key down on settings screen.
 * Implements the widget navigation through devices. 
 *
 * @param event Object with the event key press
 */
function handleKeyDownSettings(event) {

	var deviceSettings = document.getElementById(DEVICE_SETTINGS_DIV);
	var items = deviceSettings.getElementsByTagName('div');
	var selectedIndex;
	var img;
	
	// Iterate the settings items, until the selected item, and saves the item's index
	for (i = 0; i < items.length; i++) {
		if (items[i].className == 'settingsItemDivSelected') {
			selectedIndex = i;
			break;
		}
	}
	
	// Uses the index to choose the action, according to key pressed
	switch(event.keyCode) {
	
		case KEY_CODE_UP:
			if (selectedIndex != 0) {
				items[selectedIndex].className = 'settingsItemDivUnselected';
				items[selectedIndex-1].className = 'settingsItemDivSelected';
			}
			break;
		case KEY_CODE_DOWN:
			if (selectedIndex != items.length-1) {
				items[selectedIndex].className = 'settingsItemDivUnselected';
				items[selectedIndex+1].className = 'settingsItemDivSelected';
			}
			break;
		case KEY_CODE_CENTER:
			img = items[selectedIndex].getElementsByTagName('img')[0];
			if (img.checked) {
				img.src = 'resource/checkbox_unchecked.png';
				img.checked = false;
			} else {
				img.src = 'resource/checkbox_checked.png';
				img.checked = true;				
			}
				
			break;
	}
}


/**
 * Handles changes in devices status
 * 
 * @param deviceId 
 */
function handleStatusChange(deviceId) {
	
	var currentItem = menu.getCurrentItem();

	// if the status of the displayed device has changed,
	// update screen to update status value
	if (currentItem.id == deviceId) {
		switch (webui.viewType) {
			case webui.defs.viewType.LIVECON_VIEW:
				updateLiveconDeviceStatus();
				break;
			case webui.defs.viewType.DETAILED_VIEW:
				updateDetailedDeviceStatus();
				break;
		}
	}	
}


/**
 * Handles the livecon device change.
 * Flip the device on livecon view  
 */
function handleLiveconInterval() {
	menu.next();
	updateLiveconDeviceStatus();
}


/**
 * Get image index, according to status value
 * 
 * @param status
 * @return image index (1 to 4), according to status value
 */
function getImageIndexByStatus(status) {	
	var imageIndex;
	
	if (status > 80) {
		imageIndex = '1'; 
	} else if (status >= 50) {
		imageIndex = '2'; 
	} else if (status >= 20) {
		imageIndex = '3'; 
	} else {
		imageIndex = '4'; 
	}
	
	return imageIndex;	
}







